#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdint.h>
#include <cmath>
#include "PCA9685.h"

/** Default constructor, uses default I2C address.
 * @see PCA9685_DEFAULT_ADDRESS
 */
PCA9685::PCA9685() {
    devAddr = PCA9685_DEFAULT_ADDRESS;
}

/** Specific address constructor.
 * @param address I2C address
 * @see PCA9685_DEFAULT_ADDRESS
 */
PCA9685::PCA9685(uint8_t address) {
    devAddr = address;
}

bool PCA9685::reset() {
	return I2Cdev::writeByte(devAddr, PCA9685_MODE1, 0x06);
}

void PCA9685::setPWMFreq(float freq) {
	float prescaleval = 25000000;
	prescaleval /= 4096;
	prescaleval /= freq;
	prescaleval -= 1;
	uint8_t prescale = floor(prescaleval + 0.5);

	uint8_t oldmode;
	I2Cdev::readByte(devAddr, PCA9685_MODE1, &oldmode);
	uint8_t newmode = (oldmode&0x7F) | 0x10; // sleep
	I2Cdev::writeByte(devAddr, PCA9685_MODE1, newmode); // go to sleep
	I2Cdev::writeByte(devAddr, PCA9685_PRESCALE, prescale); // set the prescaler
	I2Cdev::writeByte(devAddr, PCA9685_MODE1, oldmode);
	sleep(0.005);
	I2Cdev::writeByte(devAddr, PCA9685_MODE1, oldmode | 0x80);
}

void PCA9685::setPWM(uint8_t num, uint16_t on, uint16_t off) {
	I2Cdev::writeByte(devAddr, LED0_ON_L+4*num, on);
	I2Cdev::writeByte(devAddr, LED0_ON_H+4*num, on >> 8);
	I2Cdev::writeByte(devAddr, LED0_OFF_L+4*num, off);
	I2Cdev::writeByte(devAddr, LED0_OFF_H+4*num, off >> 8);  
}

